@@ -1,19 +1,29 @@
 
 """ All the standard scripts that come with the default configuration """
 
-import sandbox, operator, os, shutil, sys
+from . import sandbox
+import operator
+import os
+import shutil
+import sys
+import random
+from glob import glob
 from texttestlib import plugins
-from ordereddict import OrderedDict
-from ConfigParser import RawConfigParser
-                    
+from collections import OrderedDict
+from configparser import RawConfigParser
+from functools import reduce
+from pprint import pformat
+
+
 class CountTest(plugins.Action):
     scriptDoc = "report on the number of tests selected, by application"
     appCount = OrderedDict()
+
     @classmethod
     def finalise(self):
-        for app, count in self.appCount.items():
-            print app.description(), "has", count, "tests"
-        print "There are", sum(self.appCount.values()), "tests in total."
+        for app, count in list(self.appCount.items()):
+            print(app.description(), "has", count, "tests")
+        print("There are", sum(self.appCount.values()), "tests in total.")
 
     def __repr__(self):
         return "Counting"
@@ -29,11 +39,47 @@ class CountTest(plugins.Action):
         self.appCount[app] = 0
 
 
+class WriteDividedSelections(plugins.ScriptWithArgs):
+    scriptDoc = "divide the test suite into equally sized selections, for parallel testing without communication possibilities"
+    files = []
+    counts = []
+
+    def __init__(self, args=[]):
+        if len(self.files) == 0:
+            WriteDividedSelections.initialise(args)
+
+    @classmethod
+    def initialise(cls, args):
+        argDict = cls.parseArguments(args, ["count", "prefix"])
+        prefix = argDict["prefix"]
+        for fn in glob(prefix + "_*"):
+            os.remove(fn)
+        for i in range(int(argDict["count"])):
+            fn = prefix + "_" + str(i + 1)
+            f = open(fn, "a")
+            f.write("-tp ")
+            cls.files.append(f)
+        cls.counts = [0] * len(cls.files)
+
+    def setUpSuite(self, suite):
+        if suite.parent is None:
+            for f in self.files:
+                f.write("appdata=" + suite.app.name + suite.app.versionSuffix() + "\n")
+
+    def __call__(self, test):
+        minCount = min(self.counts)
+        minIndices = [i for (i, count) in enumerate(self.counts) if count == minCount]
+        chosenIndex = minIndices[int(random.random() * len(minIndices))] if len(minIndices) > 1 else minIndices[0]
+        self.counts[chosenIndex] += 1
+        self.files[chosenIndex].write(test.getRelPath() + "\n")
+
+
 class DocumentOptions(plugins.Action):
-    multiValueOptions = [ "a", "c", "f", "funion", "fintersect", "t", "ts", "v" ]
+    multiValueOptions = ["a", "c", "f", "funion", "fintersect", "t", "ts", "v"]
+
     def setUpApplication(self, app):
-        groups = app.createOptionGroups([ app ])
-        keys = reduce(operator.add, (g.keys() for g in groups), [])
+        groups = app.createOptionGroups([app])
+        keys = reduce(operator.add, (list(g.keys()) for g in groups), [])
         keys.sort()
         for key in keys:
             self.displayKey(key, groups)
@@ -47,10 +93,10 @@ class DocumentOptions(plugins.Action):
 
     def display(self, keyOutput, groupOutput, docOutput):
         if not docOutput.startswith("Private"):
-            print keyOutput + ";" + groupOutput + ";" + docOutput.replace("SGE", "SGE/LSF")
+            print(keyOutput + ";" + groupOutput + ";" + docOutput.replace("SGE", "SGE/LSF"))
 
     def groupOutput(self, group):
-        knownGroups = [ "Selection", "Basic", "Advanced" ]
+        knownGroups = ["Selection", "Basic", "Advanced"]
         if group.name == "Invisible":
             return "N/A"
         elif group.name in knownGroups:
@@ -75,11 +121,11 @@ class DocumentOptions(plugins.Action):
             return keyOutput, "Select " + docs.lower()
         else:
             return keyOutput, docs
-        
+
 
 class DocumentConfig(plugins.ScriptWithArgs):
     def __init__(self, args=[]):
-        argDict = self.parseArguments(args, [ "os", "entries" ])
+        argDict = self.parseArguments(args, ["os", "entries"])
         self.onlyEntries = []
         if "entries" in argDict:
             self.onlyEntries = argDict["entries"].split(",")
@@ -89,8 +135,7 @@ class DocumentConfig(plugins.ScriptWithArgs):
         if len(self.onlyEntries) > 0:
             return self.onlyEntries
         else:
-            return sorted(app.configDir.keys() + app.configDir.aliases.keys())
-        
+            return sorted(list(app.configDir.keys()) + list(app.configDir.aliases.keys()))
 
     def reloadForOverrideOs(self, app):
         if self.overrideOs and self.overrideOs != os.name:
@@ -109,17 +154,23 @@ class DocumentConfig(plugins.ScriptWithArgs):
                 docOutput = "Alias. See entry for '" + realKey + "'"
             if not docOutput.startswith("Private"):
                 value = app.configDir[realKey]
-                print key + "|" + self.interpretArgument(value) + "|" + docOutput  
-                
+                print(key + "|" + self.interpretArgument(value) + "|" + docOutput)
+
     def interpretArgument(self, arg):
-        return str(arg).replace(plugins.installationRoots[0], "<source library>")
+        argStr = pformat(arg, width=1000) if isinstance(arg, dict) else str(arg)
+        if os.sep == "\\":
+            # in python strings get double backslashes, handle this
+            doubleBackslashRoot = plugins.installationRoots[0].replace("\\", "\\\\")
+            argStr = argStr.replace(doubleBackslashRoot, "<source library>")
+        return argStr.replace(plugins.installationRoots[0], "<source library>")
+
 
 class DocumentEnvironment(plugins.Action):
     def __init__(self, args=[]):
         self.onlyEntries = args
-        self.prefixes = [ "TEXTTEST_", "USECASE_", "STORYTEXT_" ]
-        self.exceptions = [ "TEXTTEST_PERSONAL_", "STORYTEXT_HOME_LOCAL" ]
-        
+        self.prefixes = ["TEXTTEST_", "USECASE_", "STORYTEXT_"]
+        self.exceptions = ["TEXTTEST_PERSONAL_", "STORYTEXT_HOME_LOCAL", "TEXTTEST_FAKE_USER"]
+
     def getEntriesToUse(self, app):
         rootDir = plugins.installationRoots[0]
         return self.findAllVariables(app, self.prefixes, rootDir)
@@ -134,14 +185,17 @@ class DocumentEnvironment(plugins.Action):
                 dirs.remove("storytext")
             if not includeSite and "site" in dirs:
                 dirs.remove("site")
-                
+
             # We only want to document environment variables in the configuration we're using here, not every configuration we can find
             if root.endswith("lib"):
+                toRemove = []
                 for dir in dirs:
-                    if dir != "libexec" and not sys.modules.has_key(dir) and not sys.modules.has_key("texttestlib." + dir):
-                        dirs.remove(dir)
+                    if dir != "libexec" and dir not in sys.modules and "texttestlib." + dir not in sys.modules:
+                        toRemove.append(dir)
+                for dir in toRemove:
+                    dirs.remove(dir)
             for file in files:
-                if file.endswith(".py"): 
+                if file.endswith(".py"):
                     path = os.path.join(root, file)
                     self.findVarsInFile(path, allVars, prefixes)
         return allVars
@@ -169,35 +223,31 @@ class DocumentEnvironment(plugins.Action):
         class FakeApp:
             def getConfigValue(self, name):
                 return "Config value '" + name + "'"
-        app = FakeApp() #@UnusedVariable
+        app = FakeApp()  # @UnusedVariable
         try:
             argTuple = eval(argStr)
-            from types import TupleType
-            if type(argTuple) == TupleType:
+            if type(argTuple) == tuple:
                 allArgs = list(eval(argStr))
-                return [ self.interpretArgument(str(allArgs[1])) ]
+                return [ str(allArgs[1]) ]
+            else:
+                return []
+        except Exception:  # could be anything at all
+            if argStr.endswith("executable)"):
+                return [ "<source directory>/bin/texttest" ]
             else:
                 return []
-        except Exception: # could be anything at all
-            return []
-
-    def interpretArgument(self, arg):
-        if os.path.isfile(arg) and os.path.basename(arg) in [ "texttest", "texttestc.py" ]:
-            return "<source directory>/bin/texttest"
-        else:
-            return arg
 
     def isRelevant(self, var, vars, prefixes):
         if var in self.exceptions or var in prefixes or "SLEEP" in var or "SELFTEST" in var or \
-               (len(self.onlyEntries) > 0 and var not in self.onlyEntries):
+                (len(self.onlyEntries) > 0 and var not in self.onlyEntries):
             return False
         prevVal = vars.get(var, [])
         return not prevVal or not prevVal[0]
-        
+
     def findVarsInFile(self, path, vars, prefixes):
         import re
-        regexes = [ re.compile("([^/ \"'\.,\(]*)[\(]?[\"'](" + prefix + "[^/ \"'\.,]*)") for prefix in prefixes ]
-        for line in open(path).xreadlines():
+        regexes = [re.compile("([^/ \"'\.,\(]*)[\(]?[\"'](" + prefix + "[^/ \"'\.,]*)") for prefix in prefixes]
+        for line in open(path):
             for regex in regexes:
                 match = regex.search(line)
                 if match is not None:
@@ -206,37 +256,40 @@ class DocumentEnvironment(plugins.Action):
                     if self.isRelevant(var, vars, prefixes):
                         argList = self.getArgList(line, functionName)
                         vars[var] = argList
-        
+
     def setUpApplication(self, app):
         vars = self.getEntriesToUse(app)
-        print "The following variables may be set by the user :"
+        print("The following variables may be set by the user :")
         for key in sorted(vars.keys()):
             argList = vars[key]
             if len(argList) > 1:
-                print key + "|" + "|".join(argList)
+                print(key + "|" + "|".join(argList))
 
-        print "The following variables are set by TextTest :"
-        for var in sorted(filter(lambda key: len(vars[key]) == 1, vars.keys())):
-            print var + "|" + vars[var][0]
+        print("The following variables are set by TextTest :")
+        for var in sorted([key for key in list(vars.keys()) if len(vars[key]) == 1]):
+            print(var + "|" + vars[var][0])
 
 
 class DocumentScripts(plugins.Action):
     def setUpApplication(self, app):
-        import batch, comparetest, performance
-        for module in [ batch, comparetest, sys.modules[__name__], performance ]:
+        from . import batch, comparetest, performance
+        for module in [batch, comparetest, sys.modules[__name__], performance]:
             names = dir(module)
             for name in names:
                 try:
                     docString = getattr(getattr(module, name), "scriptDoc")
-                    scriptName = module.__name__.replace("texttestlib.default.", "").replace("scripts", "default") + "." + name
-                    print scriptName + "|" + docString
+                    scriptName = module.__name__.replace("texttestlib.default.", "").replace(
+                        "scripts", "default") + "." + name
+                    print(scriptName + "|" + docString)
                 except AttributeError:
                     pass
 
+
 class ReplaceText(plugins.ScriptWithArgs):
     scriptDoc = "Perform a search and replace on all files with the given stem"
+
     def __init__(self, args):
-        argDict = self.parseArguments(args, [ "old", "new", "file", "regexp", "argsReplacement", "includeShortcuts" ])
+        argDict = self.parseArguments(args, ["old", "new", "file", "regexp", "argsReplacement", "includeShortcuts"])
         tryAsRegexp = "regexp" not in argDict or argDict["regexp"] == "1"
         self.argsReplacement = "argsReplacement" in argDict and argDict["argsReplacement"] == "1"
         self.oldText = argDict["old"].replace("\\n", "\n")
@@ -256,27 +309,36 @@ class ReplaceText(plugins.ScriptWithArgs):
         return "Replacing " + self.oldText + " with " + self.newText + " for"
 
     def getFilesToChange(self, test, stem):
-        if self.includeShortcuts and test.app.name == "shortcut":
-            return [ test.dircache.pathName(f) for f in test.dircache.contents if f.endswith(".shortcut") ]
+        if self.includeShortcuts and test.app.name == "shortcut" and test.classId() == "test-case":
+            return [test.dircache.pathName(f) for f in test.dircache.contents if f.endswith(".shortcut")]
         else:
             return test.getFileNamesMatching(stem)
 
     def __call__(self, test):
+        self.replaceInFiles(test)
+
+    def replaceInFiles(self, test):
+        replaced = False
         for stem in self.stems:
             for stdFile in self.getFilesToChange(test, stem):
                 if os.path.isfile(stdFile) and not plugins.containsAutoGeneratedText(stdFile):
                     self.replaceInFile(test, stdFile)
+                    replaced = True
+        return replaced
 
-    def replaceInFile(self, test, stdFile, stemless=False):
+    def replaceInFile(self, test, stdFile):
         fileName = os.path.basename(stdFile)
         self.describe(test, " - file " + fileName)
         sys.stdout.flush()
-        unversionedFileName = ".".join(fileName.split(".")[:2]) if not stemless else fileName
+        unversionedFileName = ".".join(fileName.split(".")[:2])
+        if "." not in unversionedFileName:
+            unversionedFileName += "." + test.app.name
         tmpFile = os.path.join(test.getDirectory(temporary=1), unversionedFileName)
         with open(tmpFile, "w") as writeFile:
             with open(stdFile) as readFile:
                 for line in readFile:
-                    writeFile.write(self.trigger.replace(line, self.newMultiLineText if not self.argsReplacement else self.replaceArgs))
+                    newLine = self.trigger.replace(line, self.newMultiLineText if not self.argsReplacement else self.replaceArgs)
+                    writeFile.write(newLine)
                 if self.oldText[-1] != "\n" and not self.argsReplacement:
                     writeFile.write(self.trigger.getLeftoverText())
 
@@ -287,8 +349,13 @@ class ReplaceText(plugins.ScriptWithArgs):
     def usesComparator(self):
         return True
 
+    def comparesSuites(self, app):
+        defStems = app.defFileStems()
+        return any((stem in defStems for stem in self.stems))
+
     def setUpSuite(self, suite):
-        self.describe(suite)
+        if not self.replaceInFiles(suite):
+            self.describe(suite)
 
     def setUpApplication(self, app):
         if len(self.stems) == 0:
@@ -296,23 +363,29 @@ class ReplaceText(plugins.ScriptWithArgs):
             if not logFile in self.stems:
                 self.stems.append(logFile)
 
+
 class ExportTests(plugins.ScriptWithArgs):
     scriptDoc = "Export the selected tests to a different test suite"
+
     def __init__(self, args):
-        argDict = self.parseArguments(args, [ "dest" ])
+        argDict = self.parseArguments(args, ["dest"])
         self.otherTTHome = argDict.get("dest")
         self.otherSuites = {}
         self.placements = {}
         if not self.otherTTHome:
-            raise plugins.TextTestError, "Must provide 'dest' argument to indicate where tests should be exported"
+            raise plugins.TextTestError("Must provide 'dest' argument to indicate where tests should be exported")
+
     def __repr__(self):
         return "Checking for export of"
+
     def __call__(self, test):
         self.tryExport(test)
+
     def setUpSuite(self, suite):
         self.placements[suite] = 0
         if suite.parent:
             self.tryExport(suite)
+
     def tryExport(self, test):
         otherRootSuite = self.otherSuites.get(test.app)
         otherTest = otherRootSuite.findSubtestWithPath(test.getRelPath())
@@ -356,28 +429,39 @@ class ExportTests(plugins.ScriptWithArgs):
         self.otherSuites[app] = app.createExtraTestSuite(otherDir=self.otherTTHome)
 
 # A standalone action, we add description and generate the main file instead...
+
+
 class ExtractStandardPerformance(sandbox.ExtractPerformanceFiles):
     scriptDoc = "update the standard performance files from the standard log files"
+
     def __init__(self):
         sandbox.ExtractPerformanceFiles.__init__(self, sandbox.MachineInfoFinder())
+
     def __repr__(self):
         return "Extracting standard performance for"
+
     def __call__(self, test):
         self.describe(test)
         sandbox.ExtractPerformanceFiles.__call__(self, test)
+
     def findLogFiles(self, test, stem):
         return test.getFileNamesMatching(stem)
+
     def getFileToWrite(self, test, stem):
         name = stem + "." + test.app.name + test.app.versionSuffix()
         return os.path.join(test.getDirectory(), name)
+
     def allMachinesTestPerformance(self, test, fileStem):
         # Assume this is OK: the current host is in any case utterly irrelevant
         return 1
+
     def setUpSuite(self, suite):
         self.describe(suite)
+
     def getMachineContents(self, test):
         return " on unknown machine (extracted)\n"
-    
+
+
 class InsertShortcuts(plugins.ScriptWithArgs):
     def __repr__(self):
         return "Inserting shortcuts into usecases for"
@@ -404,16 +488,21 @@ class InsertShortcuts(plugins.ScriptWithArgs):
     def usesComparator(self):
         return True
 
+    def comparesSuites(self, *args):
+        return False
+
+
 class FilterUIMapFile(plugins.ScriptWithArgs):
     scriptDoc = "Edit the UI map to remove all unused entries, list the unused shortcuts (GUI testing)"
     instancesByFile = OrderedDict()
+
     def __init__(self):
         self.uiMapFileHandler = None
         self.uiMapFileUsed = {}
         from storytext.replayer import ShortcutManager
         self.shortcutManager = ShortcutManager()
         self.shortcutsUsed = set()
-        
+
     def __repr__(self):
         return "Filtering UI map file for"
 
@@ -421,7 +510,7 @@ class FilterUIMapFile(plugins.ScriptWithArgs):
         storytextHome = test.getEnvironment("STORYTEXT_HOME")
         uiMapFile = os.path.join(storytextHome, "ui_map.conf")
         self.instancesByFile[uiMapFile].filterUseCaseCommands(test)
-        
+
     def storeUsage(self, script):
         for command in script.commands:
             shortcut, _ = self.shortcutManager.findShortcut(command)
@@ -448,15 +537,15 @@ class FilterUIMapFile(plugins.ScriptWithArgs):
                 self.instancesByFile[uiMapFile] = self
                 from storytext.guishared import UIMapFileHandler
                 from storytext.scriptengine import ScriptEngine
-                self.uiMapFileHandler = UIMapFileHandler([ uiMapFile ])
+                self.uiMapFileHandler = UIMapFileHandler([uiMapFile])
                 for shortcut in ScriptEngine.getShortcuts(storytextHome):
                     self.shortcutManager.add(shortcut)
         else:
             self.describe(suite)
-   
-    @classmethod     
+
+    @classmethod
     def finalise(cls):
-        for uiMapFile, instance in cls.instancesByFile.items():
+        for uiMapFile, instance in list(cls.instancesByFile.items()):
             instance.writeResults(uiMapFile)
 
     def writeResults(self, uiMapFile):
@@ -464,16 +553,17 @@ class FilterUIMapFile(plugins.ScriptWithArgs):
             if shortcut in self.shortcutsUsed:
                 self.storeUsage(shortcut)
             else:
-                print "Shortcut not used", shortcut.name
+                print("Shortcut not used", shortcut.name)
 
         writeParser = RawConfigParser(dict_type=OrderedDict)
         writeParser.optionxform = str
         for section in self.uiMapFileHandler.sections():
             usedActions = self.uiMapFileUsed.get(section, [])
-            entriesToAdd = filter(lambda (action, event): action in usedActions or len(event.strip()) == 0, self.uiMapFileHandler.items(section))
+            entriesToAdd = [action_event for action_event in self.uiMapFileHandler.items(
+                section) if action_event[0] in usedActions or len(action_event[1].strip()) == 0]
             if entriesToAdd:
                 writeParser.add_section(section)
                 for actionName, eventName in entriesToAdd:
                     writeParser.set(section, actionName, eventName)
-        print "Updating UI map file..."
+        print("Updating UI map file...")
         writeParser.write(open(uiMapFile, "w"))
