@@ -1,4 +1,3 @@
-
 """
 Code to do with the grid engine master process, i.e. submitting slave jobs and waiting for them to report back
 """
@@ -81,11 +80,17 @@ class QueueSystemServer(BaseActionRunner):
                 appCapacity = queueCapacity
             if appCapacity > 0:
                 configCapacity = app.getConfigValue("queue_system_max_capacity")
-                if configCapacity < self.maxCapacity: # accept any config capacity that has been set, i.e. is not the same as default
+                # Accept any config capacity that has been set, i.e. is not the
+                # same as default.
+                if (configCapacity is not None
+                    and configCapacity < self.maxCapacity):
                     appCapacity = configCapacity
             appCapacities.append(appCapacity)
         if all((c == 0 for c in appCapacities)):
-            raise plugins.TextTestError("The queue system module is reporting zero capacity.\nEither you have set 'queue_system_max_capacity' to 0 or something is uninstalled or unavailable. Exiting.")
+            raise plugins.TextTestError(
+                "The queue system module is reporting zero capacity.\n"
+                "Either you have set 'queue_system_max_capacity' to 0 or "
+                "something is uninstalled or unavailable. Exiting.")
 
         self.maxCapacity = min((c for c in appCapacities if c != 0))
         capacityPerSuite = self.maxCapacity / len(allApps)
@@ -96,8 +101,10 @@ class QueueSystemServer(BaseActionRunner):
     def addSuites(self, suites):
         for suite in suites:
             self.slaveLogDirs.add(suite.app.makeWriteDirectory("slavelogs"))
-            plugins.log.info("Using " + queueSystemName(suite.app) + " queues for " +
-                             suite.app.description(includeCheckout=True))
+            plugins.log.info("Using "
+                             + queueSystemName(suite.app)
+                             + " queues for "
+                             + suite.app.description(includeCheckout=True))
 
     def setSlaveServerAddress(self, address):
         self.submitAddress = os.getenv("CAPTUREMOCK_SERVER", address)
@@ -217,6 +224,17 @@ class QueueSystemServer(BaseActionRunner):
         # snap out of our loop if this was the last one. Rely on others to manage the test queue
         self.reuseFailureQueue.put(None)
 
+    def markTestReuse(self, test, newTest):
+        self.jobs[newTest] = self.getJobInfo(test)
+        with self.counterLock:
+            if self.testCount > 1:
+                self.testCount -= 1
+                postText = self.remainStr()
+            else:
+                self.submitTerminators()
+                postText = ": submitting terminators as final test" # Don't allow test count to drop to 0 here, can cause race conditions
+        self.diag.info("Reusing slave from " + test.uniqueName + " for " + newTest.uniqueName + postText)
+
     def getTestForReuse(self, test, state, tryReuse, doneRerun):
         # Pick up any test that matches the current one's resource requirements
         if not self.exited:
@@ -224,30 +242,24 @@ class QueueSystemServer(BaseActionRunner):
                 newTest = self.reusedTests.get(test)
                 newTestName = newTest.uniqueName if newTest else " no test."
                 self.diag.info("Repeating answer: using slave from " + test.uniqueName + " for " + newTestName)
-                return newTest
-            # Don't allow this to use up the terminator
-            newTest = self.getTest(block=False, replaceTerminators=True)
-            if newTest:
-                if tryReuse and self.allowReuse(test, state, newTest):
-                    if not doneRerun:
-                        self.reusedTests[test] = newTest
-                    self.jobs[newTest] = self.getJobInfo(test)
-                    with self.counterLock:
-                        if self.testCount > 1:
-                            self.testCount -= 1
-                            postText = self.remainStr()
-                        else:
-                            # Don't allow test count to drop to 0 here, can cause race conditions
-                            self.submitTerminators()
-                            postText = ": submitting terminators as final test"
-                    self.diag.info("Reusing slave from " + test.uniqueName + " for " + newTest.uniqueName + postText)
+                if newTest:
+                    self.markTestReuse(test, newTest)
                     return newTest
-                else:
-                    self.diag.info("Adding to reuse failure queue : " + newTest.uniqueName)
-                    self.reuseFailureQueue.put(newTest)
             else:
-                self.diag.info("No tests available for reuse : " + test.uniqueName)
-            self.reusedTests[test] = None
+                # Don't allow this to use up the terminator
+                newTest = self.getTest(block=False, replaceTerminators=True)
+                if newTest:
+                    if tryReuse and self.allowReuse(test, state, newTest):
+                        if not doneRerun:
+                            self.reusedTests[test] = newTest
+                        self.markTestReuse(test, newTest)
+                        return newTest
+                    else:
+                        self.diag.info("Adding to reuse failure queue : " + newTest.uniqueName)
+                        self.reuseFailureQueue.put(newTest)
+                else:
+                    self.diag.info("No tests available for reuse : " + test.uniqueName)
+                self.reusedTests[test] = None
 
         # Allowed a submitted job to terminate
         with self.counterLock:
